package at.tugraz.iaik.auth.impl

import at.tugraz.iaik.auth.mgmt.config.Validity
import at.tugraz.iaik.auth.socket.AuthSocket
import at.tugraz.iaik.auth.structs.AuthRequestListener
import at.tugraz.iaik.auth.structs.AuthResponse
import at.tugraz.iaik.auth.structs.Request
import at.tugraz.iaik.p2p.commons.CommonCrypto
import at.tugraz.iaik.p2p.commons.CryptoException
import iaik.asn1.structures.AlgorithmID
import iaik.security.ssl.CipherSuite
import java.io.IOException
import java.net.InetAddress
import java.net.Socket
import java.security.PrivateKey
import java.security.cert.X509Certificate
import java.util.*

/**
 * Created by Bernd Prünster <bernd.pruenster@iaik.tugraz.at>
 */
open class LocalAuth @Throws(IOException::class, CryptoException::class) constructor(port: Int,
                                                                                     certChain: Array<X509Certificate>,
                                                                                     cipherSuites: Array<CipherSuite>,
                                                                                     validity: Validity,
                                                                                     crlDistPoints: Array<String>? = null,
                                                                                     signingKey: PrivateKey,
                                                                                     sigAlg: AlgorithmID) : AuthSocket(
        port,
        InetAddress.getLoopbackAddress(),
        AuthResponder(signingKey, certChain[0], sigAlg, validity, crlDistPoints),
        certChain,
        cipherSuites,
        signingKey) {


    private class AuthResponder(private val signingKey: PrivateKey,
                                private val caCert: X509Certificate,
                                private val sigAlg: AlgorithmID,
                                private val validity: Validity,
                                private val crlDistPoints: Array<String>?) : AuthRequestListener {


        @Throws(CryptoException::class) override fun onRequest(src: Socket,
                                                               auth: AuthSocket,
                                                               request: Request): AuthResponse {


            val to = Date()

            to.time += validity.unit.toMillis(validity.number.toLong())
            val cert = CommonCrypto.genCert(request.pubKey,
                                            request.principal,
                                            caCert.subjectDN,
                                            signingKey,
                                            sigAlg,
                                            false,
                                            Date(),
                                            to,
                                            crlDistPoints)
            return AuthResponse(AuthResponse.Result.GRANTED, cert)
        }
    }
}