package at.tugraz.iaik.auth.io

import com.owlike.genson.Context
import com.owlike.genson.Converter
import com.owlike.genson.GensonBuilder
import com.owlike.genson.stream.ObjectReader
import com.owlike.genson.stream.ObjectWriter
import iaik.asn1.structures.Name
import java.io.IOException
import java.security.KeyFactory
import java.security.Principal
import java.security.PublicKey
import java.security.cert.X509Certificate
import java.security.spec.X509EncodedKeySpec

val GensonIO = GensonBuilder().withConverters(PubKeyConverter(), CertConverter(), PrincipalConverter()).create()!!

private class PubKeyConverter : Converter<PublicKey> {
    override fun deserialize(reader: ObjectReader, ctx: Context): PublicKey? {
        reader.beginObject()
        if (!reader.hasNext()) return null

        var type: String? = null
        var key: ByteArray? = null
        for (i in 0..1) {
            reader.next()
            if ("type" == reader.name()) type = reader.valueAsString()
            else if ("key" == reader.name()) key = reader.valueAsByteArray()
        }

        if (type == null || key == null) throw IOException("not a key!")
        reader.endObject()
        return KeyFactory.getInstance(type, "IAIK").generatePublic(X509EncodedKeySpec(key));


    }

    override fun serialize(key: PublicKey?, writer: ObjectWriter, ctx: Context) {
        if (key == null) {
            writer.beginObject()
            writer.endObject()
            return
        }
        writer.beginObject()
        writer.writeString("type", key.algorithm)
        writer.writeBytes("key", key.encoded)
        writer.endObject()
    }
}

private class CertConverter : Converter<X509Certificate> {
    override fun serialize(cert: X509Certificate?, writer: ObjectWriter, ctx: Context) {
        writer.writeBytes(cert?.encoded)

    }

    override fun deserialize(reader: ObjectReader, ctx: Context?): X509Certificate? {
        val encodedCert: ByteArray = reader.valueAsByteArray() ?: return null
        return iaik.x509.X509Certificate(encodedCert)
    }

}

private class PrincipalConverter : Converter<Principal> {
    override fun serialize(principal: Principal?, writer: ObjectWriter, ctx: Context) {
        writer.writeString(principal?.toString())

    }

    override fun deserialize(reader: ObjectReader, ctx: Context?): Principal? {
        val encodedCert: String = reader.valueAsString() ?: return null
        return Name(encodedCert)
    }

}
