package at.tugraz.iaik.auth.socket

import at.tugraz.iaik.auth.io.GensonIO
import at.tugraz.iaik.auth.ocsp.OCSPResponder
import at.tugraz.iaik.auth.structs.AuthRequestListener
import at.tugraz.iaik.auth.structs.Request
import at.tugraz.iaik.p2p.commons.CryptoException
import iaik.security.ssl.CipherSuite
import java.io.IOException
import java.net.InetAddress
import java.security.PrivateKey
import java.security.cert.X509Certificate

/**
 * Created by Bernd Prünster <bernd.pruenster@iaik.tugraz.at>
 */
abstract class AuthSocket @Throws(IOException::class, CryptoException::class) protected constructor(port: Int,
                                                                                                    address: InetAddress? = null,
                                                                                                    private val listener: AuthRequestListener,
                                                                                                    certChain: Array<X509Certificate>,
                                                                                                    cipherSuites: Array<CipherSuite>,
                                                                                                    signingKey: PrivateKey) :
        Runnable {
    val responder = OCSPResponder(signingKey, certChain)

    private var running: Boolean = false
    protected val server = TLSServerSocket(address, port, certChain, signingKey, cipherSuites, responder)


    override fun run() {
        running = true
        //server.context.setDebugStream(out)
        System.err.println("$javaClass running")
        while (running) {
            try {
                val socket = server.accept()

                val request = GensonIO.deserialize(socket.getInputStream(), Request::class.java)

                val out = (socket.getOutputStream())
                val resp = listener.onRequest(socket, this, request)
                GensonIO.serialize(resp, out)
                out.flush()
                out.close()

                socket.close()
            } catch (e: Exception) {
                e.printStackTrace()
            }

        }
    }

    @Throws(IOException::class)
    fun close() {
        running = false
        server.close()
    }

}
