package at.tugraz.iaik.auth.structs

import at.tugraz.iaik.auth.ocsp.convert
import at.tugraz.iaik.auth.socket.AuthSocket
import at.tugraz.iaik.p2p.commons.CryptoException
import com.owlike.genson.annotation.JsonProperty
import iaik.asn1.structures.AlgorithmID
import iaik.x509.X509CRL
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.net.Socket
import java.security.Principal
import java.security.PrivateKey
import java.security.PublicKey
import java.security.cert.X509Certificate
import java.util.*
import javax.ws.rs.client.ClientBuilder
import javax.ws.rs.client.Entity
import javax.ws.rs.core.MediaType

/**
 * Created by Bernd Prünster <bernd.pruenster@iaik.tugraz.at>
 */

enum class RequestType {
    AUTH,
    REVOKE
}

data class Request(@JsonProperty("pubKey") val pubKey: PublicKey, @JsonProperty("principal") val principal: Principal,
                   @JsonProperty("type") val type: RequestType)


data class AuthResponse(@JsonProperty("state") val state: Result, @JsonProperty("cert")
val cert: X509Certificate? = null) {

    enum class Result {
        GRANTED,
        DENIED
    }
}

interface AuthRequestListener {

    @Throws(CryptoException::class)
    fun onRequest(rc: Socket, auth: AuthSocket, request: Request): AuthResponse
}

class CRL(issuer: Principal, private val path: File, val crlDistPoints: Array<String>) {
    private val crl: X509CRL

    init {
        if (path.exists()) {
            val str = FileInputStream(path)
            crl = X509CRL(str)
            str.close()
        } else {
            crl = X509CRL()
            crl.issuerDN = issuer
        }
    }


    fun revoke(cert: X509Certificate, key: PrivateKey, alg: AlgorithmID): X509CRL {
        crl.addCertificate(cert.convert(), Date())
        crl.signatureAlgorithm = alg
        crl.thisUpdate = Date()
        val upd = Date();
        upd.time = System.currentTimeMillis() + 60 * 60 * 1000 * 12
        crl.nextUpdate = upd
        crl.sign(key)
        val out = FileOutputStream(path)
        out.write(crl.encoded)
        out.close()
        println(crl)
        for (c in crl.listCertificates()) for (l in c.toString().split("\n")) println("\t$l")
        for (point in crlDistPoints) {
            try {
                ClientBuilder.newClient().target(point).path("").request(MediaType.APPLICATION_OCTET_STREAM_TYPE).post(
                        Entity.entity(crl.encoded, MediaType.APPLICATION_OCTET_STREAM_TYPE))
            } catch (e: Exception) {
                System.err.println("WARN: CRL upload failed: ${e.message}")
            }
        }
        println()
        return crl
    }
}
